@@ -1,7 +1,8 @@
 import socket
 import pickle
+import struct
 from .utils import *
-from omnitools import jd_and_utf8e, utf8d, args
+from omnitools import jd_and_utf8e, utf8d, args, randb
 from easyrsa import *
 
 
@@ -18,13 +19,7 @@ class SC(object):
         rsa = EasyRSA(public_key=public_key)
         if rsa.verify(public_key, b64d(hash)):
             key = randb(256)
-            org_key = key
-            _key = []
-            max_msg_size = rsa.max_msg_size()
-            while org_key:
-                _key.append(b64e(rsa.encrypt(org_key[:max_msg_size])))
-                org_key = org_key[max_msg_size:]
-            self.request("set_key", args(*_key))
+            self.request(command="set_key", data=args(b64e(rsa.encrypt(key))))
             self.key = key
         else:
             raise Exception("current connection is under MITM attack")
@@ -37,13 +32,11 @@ class SC(object):
             request = pickle.dumps(request)
         if self.key:
             request = encrypt(self.key, request)
-        self.s.send(request)
-        while True:
-            response = utf8d(self.s.recv(1024*4))
-            if response:
-                if self.key:
-                    return decrypt(self.key, response)
-                else:
-                    return jl(response)
+        self.s.sendall(struct.pack('>I', len(request))+request)
+        response = utf8d(recv_all(self.s))
+        if self.key:
+            return decrypt(self.key, response)
+        else:
+            return jl(response)
 
 
