@@ -1,8 +1,9 @@
+import struct
 import socket
 import pickle
 import threading
 from .utils import *
-from omnitools import encryptedsocket_function, randi, p, utf8d
+from omnitools import encryptedsocket_function, p, utf8d, key_pair_format
 from debugging import *
 from easyrsa import *
 
@@ -29,16 +30,15 @@ class SS(object):
         p(f"connected\t{uid}")
         try:
             while True:
-                request = utf8d(conn.recv(1024*4))
+                request = utf8d(recv_all(conn))
                 if not request:
                     self.__key.pop(uid)
                     break
-                response = {}
                 if uid in self.__key:
                     request = decrypt(self.__key[uid], request)
                 else:
                     request = jl(request)
-                if request["command"] == "get_pkey":
+                if request["command"] == "get_pkey" and uid not in self.__key:
                     v = self.pkey()
                     response = (
                         b64e(self.sign(v)),
@@ -49,6 +49,11 @@ class SS(object):
                         response = self.functions[request["command"]](*request["data"][0], **request["data"][1])
                     except:
                         response = debug_info()
+                else:
+                    try:
+                        raise Exception("request command '{}' is not in socket functions".format(request["command"]))
+                    except:
+                        response = debug_info()
                 try:
                     from omnitools import jd_and_utf8e
                     response = jd_and_utf8e(response)
@@ -56,12 +61,10 @@ class SS(object):
                     response = pickle.dumps(response)
                 if uid in self.__key:
                     response = encrypt(self.__key[uid], response)
-                conn.sendall(response)
-                if request["command"] == "set_key":
+                conn.sendall(struct.pack('>I', len(response))+response)
+                if request["command"] == "set_key" and uid not in self.__key:
                     try:
-                        bkey = b""
-                        for part in request["data"][0]:
-                            bkey += self.rsad(b64d(part))
+                        bkey = self.rsad(b64d(request["data"][0][0]))
                     except:
                         raise Exception("current connection is under MITM attack")
                     self.__key[uid] = bkey
